//-----------------------------------------------------------------------------
//   File:      simpleio.c
//   Contents:  Firmware for VC++ software GtUSBBox.exe
//        This imprements simple I/O using Ports A,B, and D
//        Interrupt-driven input using INT0/1 is implemented.
//
//  << Caution >>
//  This firmware utilizes the Vender ID and Product ID of Cypress Semiconducter Inc..
//  Commercial use of this firmware is not allowed.
//
//-----------------------------------------------------------------------------
// Copyright 2019, Gigatec Co.,Ltd.
//-----------------------------------------------------------------------------
#pragma NOIV               // Do not generate interrupt vectors

#include "fx2.h"
#include "fx2regs.h"
#include "syncdly.h"            // SYNCDELAY macro

extern BOOL GotSUD;             // Received setup data flag
extern BOOL Sleep;
extern BOOL Rwuen;
extern BOOL Selfpwr;

BYTE Configuration;             // Current configuration
BYTE AlternateSetting;          // Alternate settings

BYTE InterruptReason;

#define VR_NAKALL_ON    0xD0
#define VR_NAKALL_OFF   0xD1

//-----------------------------------------------------------------------------
// Task Dispatcher hooks
//   The following hooks are called by the task dispatcher.
//-----------------------------------------------------------------------------

void TD_Init(void)             // Called once at startup
{
   // set the CPU clock to 48MHz
   CPUCS = ((CPUCS & ~bmCLKSPD) | bmCLKSPD1) ;

   // set the slave FIFO interface to 48MHz
   IFCONFIG |= 0x40;

  // Registers which require a synchronization delay, see section 15.14
  // FIFORESET        FIFOPINPOLAR
  // INPKTEND         OUTPKTEND
  // EPxBCH:L         REVCTL
  // GPIFTCB3         GPIFTCB2
  // GPIFTCB1         GPIFTCB0
  // EPxFIFOPFH:L     EPxAUTOINLENH:L
  // EPxFIFOCFG       EPxGPIFFLGSEL
  // PINFLAGSxx       EPxFIFOIRQ
  // EPxFIFOIE        GPIFIRQ
  // GPIFIE           GPIFADRH:L
  // UDMACRCH:L       EPxGPIFTRIG
  // GPIFTRIG
  
  // Note: The pre-REVE EPxGPIFTCH/L register are affected, as well...
  //      ...these have been replaced by GPIFTC[B3:B0] registers

  // EP1IN are Valid,Bulk.
  // default: An EP2 DIR bit is 0 (OUT direction)
  // default: An EP6 DIR bit is 1 (IN direction)
  // default: EP2 and EP6 are double buffered

  // we are just using the default values, yes this is not necessary...
  EP1INCFG = 0xA0;
  SYNCDELAY;                    // see TRM section 15.14
  EP2CFG = 0xA2;
  SYNCDELAY;                    
  EP6CFG = 0xE2;

  // out endpoints do not come up armed
  
  // since the defaults are double buffered we must write dummy byte counts twice
  SYNCDELAY;                    
  EP1OUTBC = 0x40;              // arm EP1OUT by writing byte
  SYNCDELAY;                    
  EP2BCL = 0x80;                // arm EP2OUT by writing byte count w/skip.
  SYNCDELAY;                    
  EP2BCL = 0x80;

  // enable dual autopointer feature
  AUTOPTRSETUP |= 0x01;

  IT0=IT1=1;  // INT0/INT1 is set to edge-triggered
			  // 1E0(TCON.1)/IE1(TCON.3) should be cleared after interrupt processing 
  PX0=PX1=1;  // INT0/INT1 is set to high priority
  InterruptReason=0;
}

void SetDirections(WORD count)
{
	OEA=EXTAUTODAT1;
	OEB=EXTAUTODAT1;
	OED=EXTAUTODAT1;
	if (count==6) {
		OEC=EXTAUTODAT1;
		OEE=EXTAUTODAT1;
	}
}

void GetDirections(WORD count)
{
    if(!(EP2468STAT & bmEP6FULL))
    {  // confirmed that Endpoint(0x86) is not full.
        AUTOPTRH1 = MSB( &EP6FIFOBUF );
        AUTOPTRL1 = LSB( &EP6FIFOBUF );

        EXTAUTODAT1=OEA;
        EXTAUTODAT1=OEB;
        EXTAUTODAT1=OED;
		if (count==2) {
	        EXTAUTODAT1=OEC;
   		    EXTAUTODAT1=OEE;
		} 
        EP6BCH = 0;  
        SYNCDELAY;  
        if (count==2) EP6BCL = 5;
		else EP6BCL = 3;        // arm EP6IN
    }
}

void ReadPorts(WORD count)
{
    if(!(EP2468STAT & bmEP6FULL))
    {  // confirmed that Endpoint(0x86) is not full.
        AUTOPTRH1 = MSB( &EP6FIFOBUF );
        AUTOPTRL1 = LSB( &EP6FIFOBUF );

        EXTAUTODAT1=IOA;
        EXTAUTODAT1=IOB;
        EXTAUTODAT1=IOD;
		if (count==2) {
	        EXTAUTODAT1=IOC;
   		    EXTAUTODAT1=IOE;
		} 
        EP6BCH = 0;  
        SYNCDELAY;  
        if (count==2) EP6BCL = 5;
		else EP6BCL = 3;        // arm EP6IN
    }
}

void OutputPort(WORD count, char port)
{
	if (count!=2) return;
	switch (port) {
	case 0:IOA=EXTAUTODAT1;break;
	case 1:IOB=EXTAUTODAT1;break;
	case 2:IOC=EXTAUTODAT1;break;
	case 3:IOD=EXTAUTODAT1;break;
	case 4:IOE=EXTAUTODAT1;break;
	}
}

void ActivateInterrupt(WORD count, char intn)
{
	char c;
	if (count!=1) return;
	c=(intn & 1);
	if (c && !EX0) TCON &= 0xFD;
	EX0=c; 
	c=((intn>>1) & 1);
	if (c && !EX1) TCON &= 0xF7;
	EX1=c;
}

void TD_Poll(void)              // Called repeatedly while the device is idle
{
  WORD count;
  char c;

  if(!(EP2468STAT & bmEP2EMPTY))
  { // Endpoint(0x02) has data.
     count = (EP2BCH << 8) + EP2BCL;
	 AUTOPTRH1 = MSB( &EP2FIFOBUF );
     AUTOPTRL1 = LSB( &EP2FIFOBUF );
	 c= EXTAUTODAT1;
	 switch (c & 0xf0) {
	 case 0:
		if (c & 0xf) GetDirections(count);
		else SetDirections(count);
		break;
	 case 0x10:ReadPorts(count);break;
	 case 0x20:OutputPort(count,c & 0xf);break;
	 case 0x30:ActivateInterrupt(count,c & 0xf);break;
	 }
     SYNCDELAY;                    
     EP2BCL = 0x80;          // re(arm) EP2OUT
  }
  if(InterruptReason) // External Interupt occured
  {
	 EP1INBUF[0] = InterruptReason;
     InterruptReason=0;
	 SYNCDELAY;
     EP1INBC = 1;			   // arm EP1IN
  }
}

BOOL TD_Suspend(void)          // Called before the device goes into suspend mode
{
   return(TRUE);
}

BOOL TD_Resume(void)          // Called after the device resumes
{
   return(TRUE);
}

//-----------------------------------------------------------------------------
// Device Request hooks
//   The following hooks are called by the end point 0 device request parser.
//-----------------------------------------------------------------------------

BOOL DR_GetDescriptor(void)
{
   return(TRUE);
}

BOOL DR_SetConfiguration(void)   // Called when a Set Configuration command is received
{
   Configuration = SETUPDAT[2];
   return(TRUE);            // Handled by user code
}

BOOL DR_GetConfiguration(void)   // Called when a Get Configuration command is received
{
   EP0BUF[0] = Configuration;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);            // Handled by user code
}

BOOL DR_SetInterface(void)       // Called when a Set Interface command is received
{
   AlternateSetting = SETUPDAT[2];
   return(TRUE);            // Handled by user code
}

BOOL DR_GetInterface(void)       // Called when a Set Interface command is received
{
   EP0BUF[0] = AlternateSetting;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);            // Handled by user code
}

BOOL DR_GetStatus(void)
{
   return(TRUE);
}

BOOL DR_ClearFeature(void)
{
   return(TRUE);
}

BOOL DR_SetFeature(void)
{
   return(TRUE);
}

BOOL DR_VendorCmnd(void)
{
  BYTE tmp;
  
  switch (SETUPDAT[1])
  {
     case VR_NAKALL_ON:
        tmp = FIFORESET;
        tmp |= bmNAKALL;      
        SYNCDELAY;                    
        FIFORESET = tmp;
        break;
     case VR_NAKALL_OFF:
        tmp = FIFORESET;
        tmp &= ~bmNAKALL;      
        SYNCDELAY;                    
        FIFORESET = tmp;
        break;
     default:
        return(TRUE);
  }

  return(FALSE);
}

//-----------------------------------------------------------------------------
// USB Interrupt Handlers
//   The following functions are called by the USB interrupt jump table.
//-----------------------------------------------------------------------------

// Setup Data Available Interrupt Handler
void ISR_Sudav(void) interrupt 0
{
   GotSUD = TRUE;            // Set flag
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUDAV;         // Clear SUDAV IRQ
}

// Setup Token Interrupt Handler
void ISR_Sutok(void) interrupt 0
{
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUTOK;         // Clear SUTOK IRQ
}

void ISR_Sof(void) interrupt 0
{
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSOF;            // Clear SOF IRQ
}

void ISR_Ures(void) interrupt 0
{
   // whenever we get a USB reset, we should revert to full speed mode
   pConfigDscr = pFullSpeedConfigDscr;
   ((CONFIGDSCR xdata *) pConfigDscr)->type = CONFIG_DSCR;
   pOtherConfigDscr = pHighSpeedConfigDscr;
   ((CONFIGDSCR xdata *) pOtherConfigDscr)->type = OTHERSPEED_DSCR;

   EZUSB_IRQ_CLEAR();
   USBIRQ = bmURES;         // Clear URES IRQ
}

void ISR_Susp(void) interrupt 0
{
   Sleep = TRUE;
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUSP;
}

void ISR_Highspeed(void) interrupt 0
{
   if (EZUSB_HIGHSPEED())
   {
      pConfigDscr = pHighSpeedConfigDscr;
      ((CONFIGDSCR xdata *) pConfigDscr)->type = CONFIG_DSCR;
      pOtherConfigDscr = pFullSpeedConfigDscr;
      ((CONFIGDSCR xdata *) pOtherConfigDscr)->type = OTHERSPEED_DSCR;
   }

   EZUSB_IRQ_CLEAR();
   USBIRQ = bmHSGRANT;
}
void ISR_Ep0ack(void) interrupt 0
{
}
void ISR_Stub(void) interrupt 0
{
}
void ISR_Ep0in(void) interrupt 0
{
}
void ISR_Ep0out(void) interrupt 0
{
}
void ISR_Ep1in(void) interrupt 0
{
}
void ISR_Ep1out(void) interrupt 0
{
}
void ISR_Ep2inout(void) interrupt 0
{
}
void ISR_Ep4inout(void) interrupt 0
{
}
void ISR_Ep6inout(void) interrupt 0
{
}
void ISR_Ep8inout(void) interrupt 0
{
}
void ISR_Ibn(void) interrupt 0
{
}
void ISR_Ep0pingnak(void) interrupt 0
{
}
void ISR_Ep1pingnak(void) interrupt 0
{
}
void ISR_Ep2pingnak(void) interrupt 0
{
}
void ISR_Ep4pingnak(void) interrupt 0
{
}
void ISR_Ep6pingnak(void) interrupt 0
{
}
void ISR_Ep8pingnak(void) interrupt 0
{
}
void ISR_Errorlimit(void) interrupt 0
{
}
void ISR_Ep2piderror(void) interrupt 0
{
}
void ISR_Ep4piderror(void) interrupt 0
{
}
void ISR_Ep6piderror(void) interrupt 0
{
}
void ISR_Ep8piderror(void) interrupt 0
{
}
void ISR_Ep2pflag(void) interrupt 0
{
}
void ISR_Ep4pflag(void) interrupt 0
{
}
void ISR_Ep6pflag(void) interrupt 0
{
}
void ISR_Ep8pflag(void) interrupt 0
{
}
void ISR_Ep2eflag(void) interrupt 0
{
}
void ISR_Ep4eflag(void) interrupt 0
{
}
void ISR_Ep6eflag(void) interrupt 0
{
}
void ISR_Ep8eflag(void) interrupt 0
{
}
void ISR_Ep2fflag(void) interrupt 0
{
}
void ISR_Ep4fflag(void) interrupt 0
{
}
void ISR_Ep6fflag(void) interrupt 0
{
}
void ISR_Ep8fflag(void) interrupt 0
{
}
void ISR_GpifComplete(void) interrupt 0
{
}
void ISR_GpifWaveform(void) interrupt 0
{
}

